"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stream_1 = require("stream");
const Debug = require("debug");
const statuses = require('statuses');
const qs = require('querystring');
const logger = Debug('acp:server');
const parse = require('parseurl');
function respond() {
    logger('respond called');
    const ctx = this;
    if (!ctx.writable)
        return;
    const res = ctx.res;
    let body = ctx.body;
    const code = ctx.statusCode;
    if (statuses.empty[code]) {
        ctx.body = null;
        return res.end();
    }
    if ('HEAD' === ctx.method) {
        if (!res.headersSent) {
            ctx.length = Buffer.byteLength(JSON.stringify(body));
        }
        return res.end();
    }
    if (null == body) {
        if (ctx.req.httpVersionMajor >= 2) {
            body = String(code);
        }
        else {
            body = ctx.message || String(code);
        }
        if (!res.headersSent) {
            ctx.type = 'text';
            ctx.length = Buffer.byteLength(body);
        }
        return res.end(body);
    }
    if (body instanceof stream_1.Stream)
        return body.pipe(res);
    res.end(body);
}
function CtxProvider(request, res) {
    const urlParsedCache = parse(request);
    const ctx = {
        req: request,
        get path() {
            return urlParsedCache.pathname;
        },
        get headers() {
            return this.req.headers;
        },
        get method() {
            return this.req.method;
        },
        get querystring() {
            if (!this.req)
                return '';
            return urlParsedCache.query || '';
        },
        get search() {
            if (!this.querystring)
                return '';
            return `?${this.querystring}`;
        },
        get(field) {
            const req = this.req;
            switch (field = field.toLowerCase()) {
                case 'referer':
                case 'referrer':
                    return req.headers.referrer || req.headers.referer || '';
                default:
                    return req.headers[field] || '';
            }
        },
        get query() {
            const str = this.querystring;
            const c = this._querycache = this._querycache || {};
            return c[str] || (c[str] = qs.parse(str));
        },
        res,
        body: null,
        state: {},
        get statusCode() {
            return this.res.statusCode;
        },
        set statusCode(code) {
            this.res.statusCode = code;
        },
        get statusMessage() {
            return this.res.statusMessage;
        },
        set statusMessage(msg) {
            this.res.statusMessage = msg;
        },
        respond,
        get writable() {
            if (this.res.finished)
                return false;
            const socket = this.res.socket;
            if (!socket)
                return true;
            return socket.writable;
        },
        set(field, val) {
            try {
                this.res.setHeader(field, val);
            }
            catch (e) {
                logger('Error set headers', field, val);
            }
            return this;
        },
    };
    return ctx;
}
exports.CtxProvider = CtxProvider;
